import sys
import json

import graph
import grew
import granalysis

grew.init()

g = dict()
graph.add_node(g, 'W1', {'phon' : 'the', 'cat' : 'DET'} )
graph.add_node(g, 'W2', {'phon' : 'child', 'cat' : 'N'} )
graph.add_node(g, 'W3', {'phon' : 'plays', 'cat' : 'V'} )
graph.add_node(g, 'W4', {'phon' : 'the', 'cat' : 'DET'})
graph.add_node(g, 'W5', {'phon' : 'fool', 'cat' : 'N'})
graph.add_edge(g, 'W2', 'det', 'W1')
graph.add_edge(g, 'W3', 'suj', 'W2')
graph.add_edge(g, 'W3', 'obj', 'W5')
graph.add_edge(g, 'W5', 'det', 'W4')

print("\n==================== search 'pattern {M -> N}' in 'The child plays the fool ====================\n")
matchs = grew.search ("pattern {M -> N}", g)
print ("number of matches: %d" % (len (matchs)))
print (matchs)

print("\n==================== rewrite 'Elle pense venir' with grs in 'data/test.grs' ====================\n")
ipv = dict()
graph.add_node(ipv, 'A', {'phon' : 'Elle', 'lemma':"il", 'cat' : 'PRO'} )
graph.add_node(ipv, 'B', {'phon' : 'pense', 'lemma':"penser", 'cat' : 'V'} )
graph.add_node(ipv, 'C', {'phon' : 'venir', 'lemma': 'venir', 'cat' : 'V', 'm':'inf'} )
graph.add_edge(ipv, 'B', 'suj', 'A')
graph.add_edge(ipv, 'B', 'obj', 'C')

print("\n--- before ---")
print (ipv)

print("\n--- after ---")
test_grs = grew.grs('data/test.grs')
rew = grew.run (test_grs, ipv)
print (rew)

print("\n==================== to_json ====================\n")
print  (json.dumps (grew.json_grs (test_grs), indent=2))

graph.draw(g)





"""
Grew module : anything you want to talk about graphs
Graphs are represented either by a dict (called dict-graph),
or by an str (str-graph).
"""
import os.path
import re
import copy
import tempfile
import json

from grew import network
from grew import utils

''' Library tools '''

def init():
    """
    Initialize connection to GREW library
    :return: True if anything went right
    """
    return network.init()

def grs(grsystem):
    """Load a grs stored in a file
    :param grs: either a file name or a GREW string representation of a grs
    :raise an error if the file was not correctly loaded
    """
    try:
        if os.path.isfile(grsystem):
            req = { "command": "load_grs", "filename": grsystem }
            reply = network.send_and_receive(req)
        else:
            with tempfile.NamedTemporaryFile(mode="w", delete=True, suffix=".grs") as f:
                f.write(grsystem)
                f.seek(0)  # to be read by others
                req = { "command": "load_grs", "filename": f.name }
                reply = network.send_and_receive(req)
        return reply["index"]
    except:
        raise utils.GrewError("Could not build a GRS with: %s" % grsystem)

def run(rs, gr, strat="main"):
    """
    Apply rs or the last loaded one to [gr]
    :param gr: the graph, either a str (in grew format) or a dict
    :param rs: a graph rewriting system or a GREW string representation of a grs
    :param strategy: the strategy. By default, !
    :return: the list of rewritten graphs
    """
    if isinstance(rs, int):
        index=rs
    else:
        index = grs(rs)

    req = {
        "command": "run",
        "graph": json.dumps(gr),
        "grs_index": index,
        "strat": strat
    }
    reply = network.send_and_receive(req)
    return utils.rm_dups(reply)

def corpus(data):
    """Load a corpus from a file of a string
    :param grs: either a file name or a CoNLL string representation of a corpus
    :raise an error if the file was not correctly loaded
    """
    try:
        if os.path.isfile(data):
            req = { "command": "load_corpus", "files": [data] }
            reply = network.send_and_receive(req)
        else:
            with tempfile.NamedTemporaryFile(mode="w", delete=True, suffix=".conll") as f:
                f.write(data)
                f.seek(0)  # to be read by others
                req = { "command": "load_corpus", "filename": f.name }
                reply = network.send_and_receive(req)
        return reply["index"]
    except:
        raise utils.GrewError("Could not build a corpus with: %s" % data)

def search(pattern, gr):
    """
    Search for [pattern] into [gr]
    :param patten: a string pattern
    :param gr: the graph
    :return: the list of matching of [pattern] into [gr]
    """
    req = {
        "command": "search",
        "graph": json.dumps(gr),
        "pattern": pattern
    }
    reply = network.send_and_receive(req)
    return reply

def corpus_search(pattern, corpus_index):
    """
    Search for [pattern] into [corpus_index]
    :param patten: a string pattern
    :param corpus_index: an integer given by the [corpus] function
    :return: the list of matching of [pattern] into the corpus
    """
    req = {
        "command": "corpus_search",
        "corpus_index": corpus_index,
        "pattern": pattern
    }
    reply = network.send_and_receive(req)
    return reply


def json_grs(rs):
    req = { "command": "json_grs", "grs_index": rs }
    return network.send_and_receive(req)


